package org.eclipse.jdt.internal.compiler.batch;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import org.eclipse.jdt.core.compiler.CategorizedProblem;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.IProblem;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.batch.CompilationUnit;
import org.eclipse.jdt.internal.compiler.batch.FileSystem;
import org.eclipse.jdt.internal.compiler.batch.Main;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.CompilerStats;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.util.GenericXMLWriter;
import org.eclipse.jdt.internal.compiler.util.HashtableOfInt;
import org.eclipse.jdt.internal.compiler.util.Messages;
import org.eclipse.jdt.internal.compiler.util.Util;

public class Logger {
  private PrintWriter err;
  
  private PrintWriter log;
  
  private Main main;
  
  private PrintWriter out;
  
  private HashMap parameters;
  
  int tagBits;
  
  private static final String CLASS = "class";
  
  private static final String CLASS_FILE = "classfile";
  
  private static final String CLASSPATH = "classpath";
  
  private static final String CLASSPATH_FILE = "FILE";
  
  private static final String CLASSPATH_FOLDER = "FOLDER";
  
  private static final String CLASSPATH_ID = "id";
  
  private static final String CLASSPATH_JAR = "JAR";
  
  private static final String CLASSPATHS = "classpaths";
  
  private static final String COMMAND_LINE_ARGUMENT = "argument";
  
  private static final String COMMAND_LINE_ARGUMENTS = "command_line";
  
  private static final String COMPILER = "compiler";
  
  private static final String COMPILER_COPYRIGHT = "copyright";
  
  private static final String COMPILER_NAME = "name";
  
  private static final String COMPILER_VERSION = "version";
  
  public static final int EMACS = 2;
  
  private static final String ERROR = "ERROR";
  
  private static final String ERROR_TAG = "error";
  
  private static final String WARNING_TAG = "warning";
  
  private static final String EXCEPTION = "exception";
  
  private static final String EXTRA_PROBLEM_TAG = "extra_problem";
  
  private static final String EXTRA_PROBLEMS = "extra_problems";
  
  private static final HashtableOfInt FIELD_TABLE = new HashtableOfInt();
  
  private static final String KEY = "key";
  
  private static final String MESSAGE = "message";
  
  private static final String NUMBER_OF_CLASSFILES = "number_of_classfiles";
  
  private static final String NUMBER_OF_ERRORS = "errors";
  
  private static final String NUMBER_OF_LINES = "number_of_lines";
  
  private static final String NUMBER_OF_PROBLEMS = "problems";
  
  private static final String NUMBER_OF_TASKS = "tasks";
  
  private static final String NUMBER_OF_WARNINGS = "warnings";
  
  private static final String OPTION = "option";
  
  private static final String OPTIONS = "options";
  
  private static final String OUTPUT = "output";
  
  private static final String PACKAGE = "package";
  
  private static final String PATH = "path";
  
  private static final String PROBLEM_ARGUMENT = "argument";
  
  private static final String PROBLEM_ARGUMENT_VALUE = "value";
  
  private static final String PROBLEM_ARGUMENTS = "arguments";
  
  private static final String PROBLEM_CATEGORY_ID = "categoryID";
  
  private static final String ID = "id";
  
  private static final String PROBLEM_ID = "problemID";
  
  private static final String PROBLEM_LINE = "line";
  
  private static final String PROBLEM_OPTION_KEY = "optionKey";
  
  private static final String PROBLEM_MESSAGE = "message";
  
  private static final String PROBLEM_SEVERITY = "severity";
  
  private static final String PROBLEM_SOURCE_END = "charEnd";
  
  private static final String PROBLEM_SOURCE_START = "charStart";
  
  private static final String PROBLEM_SUMMARY = "problem_summary";
  
  private static final String PROBLEM_TAG = "problem";
  
  private static final String PROBLEMS = "problems";
  
  private static final String SOURCE = "source";
  
  private static final String SOURCE_CONTEXT = "source_context";
  
  private static final String SOURCE_END = "sourceEnd";
  
  private static final String SOURCE_START = "sourceStart";
  
  private static final String SOURCES = "sources";
  
  private static final String STATS = "stats";
  
  private static final String TASK = "task";
  
  private static final String TASKS = "tasks";
  
  private static final String TIME = "time";
  
  private static final String VALUE = "value";
  
  private static final String WARNING = "WARNING";
  
  public static final int XML = 1;
  
  private static final String XML_DTD_DECLARATION = "<!DOCTYPE compiler PUBLIC \"-//Eclipse.org//DTD Eclipse JDT 3.2.004 Compiler//EN\" \"http://www.eclipse.org/jdt/core/compiler_32_004.dtd\">";
  
  static {
    try {
      Class<IProblem> c = IProblem.class;
      Field[] fields = c.getFields();
      for (int i = 0, max = fields.length; i < max; i++) {
        Field field = fields[i];
        if (field.getType().equals(int.class)) {
          Integer value = (Integer)field.get(null);
          int key2 = value.intValue() & 0xFFFFFF;
          if (key2 == 0)
            key2 = Integer.MAX_VALUE; 
          FIELD_TABLE.put(key2, field.getName());
        } 
      } 
    } catch (SecurityException e) {
      e.printStackTrace();
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } 
  }
  
  public Logger(Main main, PrintWriter out, PrintWriter err) {
    this.out = out;
    this.err = err;
    this.parameters = new HashMap<>();
    this.main = main;
  }
  
  public String buildFileName(String outputPath, String relativeFileName) {
    char fileSeparatorChar = File.separatorChar;
    String fileSeparator = File.separator;
    outputPath = outputPath.replace('/', fileSeparatorChar);
    StringBuffer outDir = new StringBuffer(outputPath);
    if (!outputPath.endsWith(fileSeparator))
      outDir.append(fileSeparator); 
    StringTokenizer tokenizer = 
      new StringTokenizer(relativeFileName, fileSeparator);
    String token = tokenizer.nextToken();
    while (tokenizer.hasMoreTokens()) {
      outDir.append(token).append(fileSeparator);
      token = tokenizer.nextToken();
    } 
    return outDir.append(token).toString();
  }
  
  public void close() {
    if (this.log != null) {
      if ((this.tagBits & 0x1) != 0) {
        endTag("compiler");
        flush();
      } 
      this.log.close();
    } 
  }
  
  public void compiling() {
    printlnOut(this.main.bind("progress.compiling"));
  }
  
  private void endLoggingExtraProblems() {
    endTag("extra_problems");
  }
  
  private void endLoggingProblems() {
    endTag("problems");
  }
  
  public void endLoggingSource() {
    if ((this.tagBits & 0x1) != 0)
      endTag("source"); 
  }
  
  public void endLoggingSources() {
    if ((this.tagBits & 0x1) != 0)
      endTag("sources"); 
  }
  
  public void endLoggingTasks() {
    if ((this.tagBits & 0x1) != 0)
      endTag("tasks"); 
  }
  
  private void endTag(String name) {
    if (this.log != null)
      ((GenericXMLWriter)this.log).endTag(name, true, true); 
  }
  
  private String errorReportSource(CategorizedProblem problem, char[] unitSource, int bits) {
    int startPosition = problem.getSourceStart();
    int endPosition = problem.getSourceEnd();
    if (unitSource == null && 
      problem.getOriginatingFileName() != null)
      try {
        unitSource = Util.getFileCharContent(new File(new String(problem.getOriginatingFileName())), null);
      } catch (IOException iOException) {} 
    int length;
    if (startPosition > endPosition || (
      startPosition < 0 && endPosition < 0) || 
      unitSource == null || (
      length = unitSource.length) == 0)
      return Messages.problem_noSourceInformation; 
    StringBuffer errorBuffer = new StringBuffer();
    if ((bits & 0x2) == 0) {
      errorBuffer.append(' ').append(Messages.bind(Messages.problem_atLine, String.valueOf(problem.getSourceLineNumber())));
      errorBuffer.append(Util.LINE_SEPARATOR);
    } 
    errorBuffer.append('\t');
    char c;
    int begin;
    for (begin = (startPosition >= length) ? (length - 1) : startPosition; begin > 0 && (
      c = unitSource[begin - 1]) != '\n' && c != '\r'; begin--);
    int end;
    for (end = (endPosition >= length) ? (length - 1) : endPosition; end + 1 < length && (
      c = unitSource[end + 1]) != '\r' && c != '\n'; end++);
    for (; (c = unitSource[begin]) == ' ' || c == '\t'; begin++);
    errorBuffer.append(unitSource, begin, end - begin + 1);
    errorBuffer.append(Util.LINE_SEPARATOR).append("\t");
    int i;
    for (i = begin; i < startPosition; i++)
      errorBuffer.append((unitSource[i] == '\t') ? 9 : 32); 
    for (i = startPosition; i <= ((endPosition >= length) ? (length - 1) : endPosition); i++)
      errorBuffer.append('^'); 
    return errorBuffer.toString();
  }
  
  private void extractContext(CategorizedProblem problem, char[] unitSource) {
    int startPosition = problem.getSourceStart();
    int endPosition = problem.getSourceEnd();
    if (unitSource == null && 
      problem.getOriginatingFileName() != null)
      try {
        unitSource = Util.getFileCharContent(new File(new String(problem.getOriginatingFileName())), null);
      } catch (IOException iOException) {} 
    int length;
    if (startPosition > endPosition || (
      startPosition < 0 && endPosition < 0) || 
      unitSource == null || (
      length = unitSource.length) <= 0 || 
      endPosition > length) {
      this.parameters.put("value", Messages.problem_noSourceInformation);
      this.parameters.put("sourceStart", "-1");
      this.parameters.put("sourceEnd", "-1");
      printTag("source_context", this.parameters, true, true);
      return;
    } 
    char c;
    int begin;
    for (begin = (startPosition >= length) ? (length - 1) : startPosition; begin > 0 && (
      c = unitSource[begin - 1]) != '\n' && c != '\r'; begin--);
    int end;
    for (end = (endPosition >= length) ? (length - 1) : endPosition; end + 1 < length && (
      c = unitSource[end + 1]) != '\r' && c != '\n'; end++);
    for (; (c = unitSource[begin]) == ' ' || c == '\t'; begin++);
    for (; (c = unitSource[end]) == ' ' || c == '\t'; end--);
    StringBuffer buffer = new StringBuffer();
    buffer.append(unitSource, begin, end - begin + 1);
    this.parameters.put("value", String.valueOf(buffer));
    this.parameters.put("sourceStart", Integer.toString(startPosition - begin));
    this.parameters.put("sourceEnd", Integer.toString(endPosition - begin));
    printTag("source_context", this.parameters, true, true);
  }
  
  public void flush() {
    this.out.flush();
    this.err.flush();
    if (this.log != null)
      this.log.flush(); 
  }
  
  private String getFieldName(int id) {
    int key2 = id & 0xFFFFFF;
    if (key2 == 0)
      key2 = Integer.MAX_VALUE; 
    return (String)FIELD_TABLE.get(key2);
  }
  
  private String getProblemOptionKey(int problemID) {
    int irritant = ProblemReporter.getIrritant(problemID);
    return CompilerOptions.optionKeyFromIrritant(irritant);
  }
  
  public void logAverage() {
    Arrays.sort((Object[])this.main.compilerStats);
    long lineCount = (this.main.compilerStats[0]).lineCount;
    int length = this.main.maxRepetition;
    long sum = 0L;
    long parseSum = 0L, resolveSum = 0L, analyzeSum = 0L, generateSum = 0L;
    for (int i = 1, max = length - 1; i < max; i++) {
      CompilerStats stats = this.main.compilerStats[i];
      sum += stats.elapsedTime();
      parseSum += stats.parseTime;
      resolveSum += stats.resolveTime;
      analyzeSum += stats.analyzeTime;
      generateSum += stats.generateTime;
    } 
    long time = sum / (length - 2);
    long parseTime = parseSum / (length - 2);
    long resolveTime = resolveSum / (length - 2);
    long analyzeTime = analyzeSum / (length - 2);
    long generateTime = generateSum / (length - 2);
    printlnOut(this.main.bind(
          "compile.averageTime", 
          new String[] { String.valueOf(lineCount), 
            String.valueOf(time), 
            String.valueOf((int)(lineCount * 10000.0D / time) / 10.0D) }));
    if ((this.main.timing & 0x2) != 0)
      printlnOut(
          this.main.bind("compile.detailedTime", 
            new String[] { String.valueOf(parseTime), 
              String.valueOf((int)(parseTime * 1000.0D / time) / 10.0D), 
              String.valueOf(resolveTime), 
              String.valueOf((int)(resolveTime * 1000.0D / time) / 10.0D), 
              String.valueOf(analyzeTime), 
              String.valueOf((int)(analyzeTime * 1000.0D / time) / 10.0D), 
              String.valueOf(generateTime), 
              String.valueOf((int)(generateTime * 1000.0D / time) / 10.0D) })); 
  }
  
  public void logClassFile(boolean generatePackagesStructure, String outputPath, String relativeFileName) {
    if ((this.tagBits & 0x1) != 0) {
      String fileName = null;
      if (generatePackagesStructure) {
        fileName = buildFileName(outputPath, relativeFileName);
      } else {
        char fileSeparatorChar = File.separatorChar;
        String fileSeparator = File.separator;
        outputPath = outputPath.replace('/', fileSeparatorChar);
        int indexOfPackageSeparator = relativeFileName.lastIndexOf(fileSeparatorChar);
        if (indexOfPackageSeparator == -1) {
          if (outputPath.endsWith(fileSeparator)) {
            fileName = String.valueOf(outputPath) + relativeFileName;
          } else {
            fileName = String.valueOf(outputPath) + fileSeparator + relativeFileName;
          } 
        } else {
          int length = relativeFileName.length();
          if (outputPath.endsWith(fileSeparator)) {
            fileName = String.valueOf(outputPath) + relativeFileName.substring(indexOfPackageSeparator + 1, length);
          } else {
            fileName = String.valueOf(outputPath) + fileSeparator + relativeFileName.substring(indexOfPackageSeparator + 1, length);
          } 
        } 
      } 
      File f = new File(fileName);
      try {
        this.parameters.put("path", f.getCanonicalPath());
        printTag("classfile", this.parameters, true, true);
      } catch (IOException e) {
        logNoClassFileCreated(outputPath, relativeFileName, e);
      } 
    } 
  }
  
  public void logClasspath(FileSystem.Classpath[] classpaths) {
    if (classpaths == null)
      return; 
    if ((this.tagBits & 0x1) != 0) {
      int length = classpaths.length;
      if (length != 0) {
        printTag("classpaths", null, true, false);
        for (int i = 0; i < length; i++) {
          String classpath = classpaths[i].getPath();
          this.parameters.put("path", classpath);
          File f = new File(classpath);
          String id = null;
          if (f.isFile()) {
            if (Util.isPotentialZipArchive(classpath)) {
              id = "JAR";
            } else {
              id = "FILE";
            } 
          } else if (f.isDirectory()) {
            id = "FOLDER";
          } 
          if (id != null) {
            this.parameters.put("id", id);
            printTag("classpath", this.parameters, true, true);
          } 
        } 
        endTag("classpaths");
      } 
    } 
  }
  
  public void logCommandLineArguments(String[] commandLineArguments) {
    if (commandLineArguments == null)
      return; 
    if ((this.tagBits & 0x1) != 0) {
      int length = commandLineArguments.length;
      if (length != 0) {
        printTag("command_line", null, true, false);
        for (int i = 0; i < length; i++) {
          this.parameters.put("value", commandLineArguments[i]);
          printTag("argument", this.parameters, true, true);
        } 
        endTag("command_line");
      } 
    } 
  }
  
  public void logException(Exception e) {
    // Byte code:
    //   0: new java/io/StringWriter
    //   3: dup
    //   4: invokespecial <init> : ()V
    //   7: astore_2
    //   8: new java/io/PrintWriter
    //   11: dup
    //   12: aload_2
    //   13: invokespecial <init> : (Ljava/io/Writer;)V
    //   16: astore_3
    //   17: aload_1
    //   18: aload_3
    //   19: invokevirtual printStackTrace : (Ljava/io/PrintWriter;)V
    //   22: aload_3
    //   23: invokevirtual flush : ()V
    //   26: aload_3
    //   27: invokevirtual close : ()V
    //   30: aload_2
    //   31: invokevirtual toString : ()Ljava/lang/String;
    //   34: astore #4
    //   36: aload_0
    //   37: getfield tagBits : I
    //   40: iconst_1
    //   41: iand
    //   42: ifeq -> 191
    //   45: new java/io/LineNumberReader
    //   48: dup
    //   49: new java/io/StringReader
    //   52: dup
    //   53: aload #4
    //   55: invokespecial <init> : (Ljava/lang/String;)V
    //   58: invokespecial <init> : (Ljava/io/Reader;)V
    //   61: astore #5
    //   63: iconst_0
    //   64: istore #7
    //   66: new java/lang/StringBuffer
    //   69: dup
    //   70: invokespecial <init> : ()V
    //   73: astore #8
    //   75: aload_1
    //   76: invokevirtual getMessage : ()Ljava/lang/String;
    //   79: astore #9
    //   81: aload #9
    //   83: ifnull -> 120
    //   86: aload #8
    //   88: aload #9
    //   90: invokevirtual append : (Ljava/lang/String;)Ljava/lang/StringBuffer;
    //   93: getstatic org/eclipse/jdt/internal/compiler/util/Util.LINE_SEPARATOR : Ljava/lang/String;
    //   96: invokevirtual append : (Ljava/lang/String;)Ljava/lang/StringBuffer;
    //   99: pop
    //   100: goto -> 120
    //   103: aload #8
    //   105: aload #6
    //   107: invokevirtual append : (Ljava/lang/String;)Ljava/lang/StringBuffer;
    //   110: getstatic org/eclipse/jdt/internal/compiler/util/Util.LINE_SEPARATOR : Ljava/lang/String;
    //   113: invokevirtual append : (Ljava/lang/String;)Ljava/lang/StringBuffer;
    //   116: pop
    //   117: iinc #7, 1
    //   120: aload #5
    //   122: invokevirtual readLine : ()Ljava/lang/String;
    //   125: dup
    //   126: astore #6
    //   128: ifnull -> 137
    //   131: iload #7
    //   133: iconst_4
    //   134: if_icmplt -> 103
    //   137: aload #5
    //   139: invokevirtual close : ()V
    //   142: goto -> 146
    //   145: pop
    //   146: aload #8
    //   148: invokevirtual toString : ()Ljava/lang/String;
    //   151: astore #9
    //   153: aload_0
    //   154: getfield parameters : Ljava/util/HashMap;
    //   157: ldc 'message'
    //   159: aload #9
    //   161: invokevirtual put : (Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;
    //   164: pop
    //   165: aload_0
    //   166: getfield parameters : Ljava/util/HashMap;
    //   169: ldc 'class'
    //   171: aload_1
    //   172: invokevirtual getClass : ()Ljava/lang/Class;
    //   175: invokevirtual put : (Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;
    //   178: pop
    //   179: aload_0
    //   180: ldc 'exception'
    //   182: aload_0
    //   183: getfield parameters : Ljava/util/HashMap;
    //   186: iconst_1
    //   187: iconst_1
    //   188: invokespecial printTag : (Ljava/lang/String;Ljava/util/HashMap;ZZ)V
    //   191: aload_1
    //   192: invokevirtual getMessage : ()Ljava/lang/String;
    //   195: astore #5
    //   197: aload #5
    //   199: ifnonnull -> 211
    //   202: aload_0
    //   203: aload #4
    //   205: invokespecial printlnErr : (Ljava/lang/String;)V
    //   208: goto -> 217
    //   211: aload_0
    //   212: aload #5
    //   214: invokespecial printlnErr : (Ljava/lang/String;)V
    //   217: return
    // Line number table:
    //   Java source line number -> byte code offset
    //   #548	-> 0
    //   #549	-> 8
    //   #550	-> 17
    //   #551	-> 22
    //   #552	-> 26
    //   #553	-> 30
    //   #554	-> 36
    //   #555	-> 45
    //   #557	-> 63
    //   #558	-> 66
    //   #559	-> 75
    //   #560	-> 81
    //   #561	-> 86
    //   #564	-> 100
    //   #565	-> 103
    //   #566	-> 117
    //   #564	-> 120
    //   #568	-> 137
    //   #569	-> 142
    //   #572	-> 146
    //   #573	-> 153
    //   #574	-> 165
    //   #575	-> 179
    //   #577	-> 191
    //   #578	-> 197
    //   #579	-> 202
    //   #580	-> 208
    //   #581	-> 211
    //   #583	-> 217
    // Local variable table:
    //   start	length	slot	name	descriptor
    //   0	218	0	this	Lorg/eclipse/jdt/internal/compiler/batch/Main$Logger;
    //   0	218	1	e	Ljava/lang/Exception;
    //   8	210	2	writer	Ljava/io/StringWriter;
    //   17	201	3	printWriter	Ljava/io/PrintWriter;
    //   36	182	4	stackTrace	Ljava/lang/String;
    //   63	128	5	reader	Ljava/io/LineNumberReader;
    //   103	17	6	line	Ljava/lang/String;
    //   128	17	6	line	Ljava/lang/String;
    //   66	125	7	i	I
    //   75	116	8	buffer	Ljava/lang/StringBuffer;
    //   81	110	9	message	Ljava/lang/String;
    //   197	21	5	message	Ljava/lang/String;
    // Exception table:
    //   from	to	target	type
    //   100	142	145	java/io/IOException
  }
  
  private void logExtraProblem(CategorizedProblem problem, int localErrorCount, int globalErrorCount) {
    char[] originatingFileName = problem.getOriginatingFileName();
    if (originatingFileName == null) {
      if (problem.isError()) {
        printErr(this.main.bind(
              "requestor.extraerror", 
              Integer.toString(globalErrorCount)));
      } else {
        printErr(this.main.bind(
              "requestor.extrawarning", 
              Integer.toString(globalErrorCount)));
      } 
      printErr(" ");
      printlnErr(problem.getMessage());
    } else {
      String fileName = new String(originatingFileName);
      if ((this.tagBits & 0x2) != 0) {
        String result = String.valueOf(fileName) + 
          ":" + 
          problem.getSourceLineNumber() + 
          ": " + (
          problem.isError() ? this.main.bind("output.emacs.error") : this.main.bind("output.emacs.warning")) + 
          ": " + 
          problem.getMessage();
        printlnErr(result);
        String errorReportSource = errorReportSource(problem, null, this.tagBits);
        printlnErr(errorReportSource);
      } else {
        if (localErrorCount == 0)
          printlnErr("----------"); 
        printErr(problem.isError() ? 
            this.main.bind(
              "requestor.error", 
              Integer.toString(globalErrorCount), 
              new String(fileName)) : 
            this.main.bind(
              "requestor.warning", 
              Integer.toString(globalErrorCount), 
              new String(fileName)));
        String errorReportSource = errorReportSource(problem, null, 0);
        printlnErr(errorReportSource);
        printlnErr(problem.getMessage());
        printlnErr("----------");
      } 
    } 
  }
  
  public void loggingExtraProblems(Main currentMain) {
    ArrayList<CategorizedProblem> problems = currentMain.extraProblems;
    int count = problems.size();
    int localProblemCount = 0;
    if (count != 0) {
      int errors = 0;
      int warnings = 0;
      int i;
      for (i = 0; i < count; i++) {
        CategorizedProblem problem = problems.get(i);
        if (problem != null) {
          currentMain.globalProblemsCount++;
          logExtraProblem(problem, localProblemCount, currentMain.globalProblemsCount);
          localProblemCount++;
          if (problem.isError()) {
            errors++;
            currentMain.globalErrorsCount++;
          } else if (problem.isWarning()) {
            currentMain.globalWarningsCount++;
            warnings++;
          } 
        } 
      } 
      if ((this.tagBits & 0x1) != 0 && 
        errors + warnings != 0) {
        startLoggingExtraProblems(count);
        for (i = 0; i < count; i++) {
          CategorizedProblem problem = problems.get(i);
          if (problem != null && 
            problem.getID() != 536871362)
            logXmlExtraProblem(problem, localProblemCount, currentMain.globalProblemsCount); 
        } 
        endLoggingExtraProblems();
      } 
    } 
  }
  
  public void logUnavaibleAPT(String className) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("message", this.main.bind("configure.unavailableAPT", className));
      printTag("error", this.parameters, true, true);
    } 
    printlnErr(this.main.bind("configure.unavailableAPT", className));
  }
  
  public void logIncorrectVMVersionForAnnotationProcessing() {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("message", this.main.bind("configure.incorrectVMVersionforAPT"));
      printTag("error", this.parameters, true, true);
    } 
    printlnErr(this.main.bind("configure.incorrectVMVersionforAPT"));
  }
  
  public void logNoClassFileCreated(String outputDir, String relativeFileName, IOException e) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("message", this.main.bind("output.noClassFileCreated", 
            new String[] { outputDir, 
              relativeFileName, 
              e.getMessage() }));
      printTag("error", this.parameters, true, true);
    } 
    printlnErr(this.main.bind("output.noClassFileCreated", 
          new String[] { outputDir, 
            relativeFileName, 
            e.getMessage() }));
  }
  
  public void logNumberOfClassFilesGenerated(int exportedClassFilesCounter) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("value", Integer.valueOf(exportedClassFilesCounter));
      printTag("number_of_classfiles", this.parameters, true, true);
    } 
    if (exportedClassFilesCounter == 1) {
      printlnOut(this.main.bind("compile.oneClassFileGenerated"));
    } else {
      printlnOut(this.main.bind("compile.severalClassFilesGenerated", 
            String.valueOf(exportedClassFilesCounter)));
    } 
  }
  
  public void logOptions(Map<String, String> options) {
    if ((this.tagBits & 0x1) != 0) {
      printTag("options", null, true, false);
      Set<Map.Entry<String, String>> entriesSet = options.entrySet();
      Map.Entry[] entries = entriesSet.<Map.Entry>toArray(new Map.Entry[entriesSet.size()]);
      Arrays.sort(entries, (Comparator<? super Map.Entry>)new Object(this));
      for (int i = 0, max = entries.length; i < max; i++) {
        Map.Entry<String, String> entry = entries[i];
        String key = entry.getKey();
        this.parameters.put("key", key);
        this.parameters.put("value", entry.getValue());
        printTag("option", this.parameters, true, true);
      } 
      endTag("options");
    } 
  }
  
  public void logPendingError(String error) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("message", error);
      printTag("error", this.parameters, true, true);
    } 
    printlnErr(error);
  }
  
  public void logWarning(String message) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("message", message);
      printTag("warning", this.parameters, true, true);
    } 
    printlnOut(message);
  }
  
  private void logProblem(CategorizedProblem problem, int localErrorCount, int globalErrorCount, char[] unitSource) {
    if ((this.tagBits & 0x2) != 0) {
      String result = String.valueOf(new String(problem.getOriginatingFileName())) + 
        ":" + 
        problem.getSourceLineNumber() + 
        ": " + (
        problem.isError() ? this.main.bind("output.emacs.error") : this.main.bind("output.emacs.warning")) + 
        ": " + 
        problem.getMessage();
      printlnErr(result);
      String errorReportSource = errorReportSource(problem, unitSource, this.tagBits);
      if (errorReportSource.length() != 0)
        printlnErr(errorReportSource); 
    } else {
      if (localErrorCount == 0)
        printlnErr("----------"); 
      printErr(problem.isError() ? 
          this.main.bind(
            "requestor.error", 
            Integer.toString(globalErrorCount), 
            new String(problem.getOriginatingFileName())) : 
          this.main.bind(
            "requestor.warning", 
            Integer.toString(globalErrorCount), 
            new String(problem.getOriginatingFileName())));
      try {
        String errorReportSource = errorReportSource(problem, unitSource, 0);
        printlnErr(errorReportSource);
        printlnErr(problem.getMessage());
      } catch (Exception exception) {
        printlnErr(this.main.bind(
              "requestor.notRetrieveErrorMessage", problem.toString()));
      } 
      printlnErr("----------");
    } 
  }
  
  public int logProblems(CategorizedProblem[] problems, char[] unitSource, Main currentMain) {
    int count = problems.length;
    int localErrorCount = 0;
    int localProblemCount = 0;
    if (count != 0) {
      int errors = 0;
      int warnings = 0;
      int tasks = 0;
      int i;
      for (i = 0; i < count; i++) {
        CategorizedProblem problem = problems[i];
        if (problem != null) {
          currentMain.globalProblemsCount++;
          logProblem(problem, localProblemCount, currentMain.globalProblemsCount, unitSource);
          localProblemCount++;
          if (problem.isError()) {
            localErrorCount++;
            errors++;
            currentMain.globalErrorsCount++;
          } else if (problem.getID() == 536871362) {
            currentMain.globalTasksCount++;
            tasks++;
          } else {
            currentMain.globalWarningsCount++;
            warnings++;
          } 
        } 
      } 
      if ((this.tagBits & 0x1) != 0) {
        if (errors + warnings != 0) {
          startLoggingProblems(errors, warnings);
          for (i = 0; i < count; i++) {
            CategorizedProblem problem = problems[i];
            if (problem != null && 
              problem.getID() != 536871362)
              logXmlProblem(problem, unitSource); 
          } 
          endLoggingProblems();
        } 
        if (tasks != 0) {
          startLoggingTasks(tasks);
          for (i = 0; i < count; i++) {
            CategorizedProblem problem = problems[i];
            if (problem != null && 
              problem.getID() == 536871362)
              logXmlTask(problem, unitSource); 
          } 
          endLoggingTasks();
        } 
      } 
    } 
    return localErrorCount;
  }
  
  public void logProblemsSummary(int globalProblemsCount, int globalErrorsCount, int globalWarningsCount, int globalTasksCount) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("problems", Integer.valueOf(globalProblemsCount));
      this.parameters.put("errors", Integer.valueOf(globalErrorsCount));
      this.parameters.put("warnings", Integer.valueOf(globalWarningsCount));
      this.parameters.put("tasks", Integer.valueOf(globalTasksCount));
      printTag("problem_summary", this.parameters, true, true);
    } 
    if (globalProblemsCount == 1) {
      String message = null;
      if (globalErrorsCount == 1) {
        message = this.main.bind("compile.oneError");
      } else {
        message = this.main.bind("compile.oneWarning");
      } 
      printErr(this.main.bind("compile.oneProblem", message));
    } else {
      String errorMessage = null;
      String warningMessage = null;
      if (globalErrorsCount > 0)
        if (globalErrorsCount == 1) {
          errorMessage = this.main.bind("compile.oneError");
        } else {
          errorMessage = this.main.bind("compile.severalErrors", String.valueOf(globalErrorsCount));
        }  
      int warningsNumber = globalWarningsCount + globalTasksCount;
      if (warningsNumber > 0)
        if (warningsNumber == 1) {
          warningMessage = this.main.bind("compile.oneWarning");
        } else {
          warningMessage = this.main.bind("compile.severalWarnings", String.valueOf(warningsNumber));
        }  
      if (errorMessage == null || warningMessage == null) {
        if (errorMessage == null) {
          printErr(this.main.bind(
                "compile.severalProblemsErrorsOrWarnings", 
                String.valueOf(globalProblemsCount), 
                warningMessage));
        } else {
          printErr(this.main.bind(
                "compile.severalProblemsErrorsOrWarnings", 
                String.valueOf(globalProblemsCount), 
                errorMessage));
        } 
      } else {
        printErr(this.main.bind(
              "compile.severalProblemsErrorsAndWarnings", 
              new String[] { String.valueOf(globalProblemsCount), 
                errorMessage, 
                warningMessage }));
      } 
    } 
    if ((this.tagBits & 0x1) == 0)
      printlnErr(); 
  }
  
  public void logProgress() {
    printOut('.');
  }
  
  public void logRepetition(int i, int repetitions) {
    printlnOut(this.main.bind("compile.repetition", 
          String.valueOf(i + 1), String.valueOf(repetitions)));
  }
  
  public void logTiming(CompilerStats compilerStats) {
    long time = compilerStats.elapsedTime();
    long lineCount = compilerStats.lineCount;
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("value", Long.valueOf(time));
      printTag("time", this.parameters, true, true);
      this.parameters.put("value", Long.valueOf(lineCount));
      printTag("number_of_lines", this.parameters, true, true);
    } 
    if (lineCount != 0L) {
      printlnOut(
          this.main.bind("compile.instantTime", 
            new String[] { String.valueOf(lineCount), 
              String.valueOf(time), 
              String.valueOf((int)(lineCount * 10000.0D / time) / 10.0D) }));
    } else {
      printlnOut(
          this.main.bind("compile.totalTime", 
            new String[] { String.valueOf(time) }));
    } 
    if ((this.main.timing & 0x2) != 0)
      printlnOut(
          this.main.bind("compile.detailedTime", 
            new String[] { String.valueOf(compilerStats.parseTime), 
              String.valueOf((int)(compilerStats.parseTime * 1000.0D / time) / 10.0D), 
              String.valueOf(compilerStats.resolveTime), 
              String.valueOf((int)(compilerStats.resolveTime * 1000.0D / time) / 10.0D), 
              String.valueOf(compilerStats.analyzeTime), 
              String.valueOf((int)(compilerStats.analyzeTime * 1000.0D / time) / 10.0D), 
              String.valueOf(compilerStats.generateTime), 
              String.valueOf((int)(compilerStats.generateTime * 1000.0D / time) / 10.0D) })); 
  }
  
  public void logUsage(String usage) {
    printlnOut(usage);
  }
  
  public void logVersion(boolean printToOut) {
    if (this.log != null && (this.tagBits & 0x1) == 0) {
      String version = this.main.bind("misc.version", 
          new String[] { this.main.bind("compiler.name"), 
            this.main.bind("compiler.version"), 
            this.main.bind("compiler.copyright") });
      this.log.println("# " + version);
      if (printToOut) {
        this.out.println(version);
        this.out.flush();
      } 
    } else if (printToOut) {
      String version = this.main.bind("misc.version", 
          new String[] { this.main.bind("compiler.name"), 
            this.main.bind("compiler.version"), 
            this.main.bind("compiler.copyright") });
      this.out.println(version);
      this.out.flush();
    } 
  }
  
  public void logWrongJDK() {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("message", this.main.bind("configure.requiresJDK1.2orAbove"));
      printTag("ERROR", this.parameters, true, true);
    } 
    printlnErr(this.main.bind("configure.requiresJDK1.2orAbove"));
  }
  
  private void logXmlExtraProblem(CategorizedProblem problem, int globalErrorCount, int localErrorCount) {
    int sourceStart = problem.getSourceStart();
    int sourceEnd = problem.getSourceEnd();
    boolean isError = problem.isError();
    this.parameters.put("severity", isError ? "ERROR" : "WARNING");
    this.parameters.put("line", Integer.valueOf(problem.getSourceLineNumber()));
    this.parameters.put("charStart", Integer.valueOf(sourceStart));
    this.parameters.put("charEnd", Integer.valueOf(sourceEnd));
    printTag("extra_problem", this.parameters, true, false);
    this.parameters.put("value", problem.getMessage());
    printTag("message", this.parameters, true, true);
    extractContext(problem, null);
    endTag("extra_problem");
  }
  
  private void logXmlProblem(CategorizedProblem problem, char[] unitSource) {
    int sourceStart = problem.getSourceStart();
    int sourceEnd = problem.getSourceEnd();
    int id = problem.getID();
    this.parameters.put("id", getFieldName(id));
    this.parameters.put("problemID", Integer.valueOf(id));
    boolean isError = problem.isError();
    int severity = isError ? 1 : 0;
    this.parameters.put("severity", isError ? "ERROR" : "WARNING");
    this.parameters.put("line", Integer.valueOf(problem.getSourceLineNumber()));
    this.parameters.put("charStart", Integer.valueOf(sourceStart));
    this.parameters.put("charEnd", Integer.valueOf(sourceEnd));
    String problemOptionKey = getProblemOptionKey(id);
    if (problemOptionKey != null)
      this.parameters.put("optionKey", problemOptionKey); 
    int categoryID = ProblemReporter.getProblemCategory(severity, id);
    this.parameters.put("categoryID", Integer.valueOf(categoryID));
    printTag("problem", this.parameters, true, false);
    this.parameters.put("value", problem.getMessage());
    printTag("message", this.parameters, true, true);
    extractContext(problem, unitSource);
    String[] arguments = problem.getArguments();
    int length = arguments.length;
    if (length != 0) {
      printTag("arguments", null, true, false);
      for (int i = 0; i < length; i++) {
        this.parameters.put("value", arguments[i]);
        printTag("argument", this.parameters, true, true);
      } 
      endTag("arguments");
    } 
    endTag("problem");
  }
  
  private void logXmlTask(CategorizedProblem problem, char[] unitSource) {
    this.parameters.put("line", Integer.valueOf(problem.getSourceLineNumber()));
    this.parameters.put("charStart", Integer.valueOf(problem.getSourceStart()));
    this.parameters.put("charEnd", Integer.valueOf(problem.getSourceEnd()));
    String problemOptionKey = getProblemOptionKey(problem.getID());
    if (problemOptionKey != null)
      this.parameters.put("optionKey", problemOptionKey); 
    printTag("task", this.parameters, true, false);
    this.parameters.put("value", problem.getMessage());
    printTag("message", this.parameters, true, true);
    extractContext(problem, unitSource);
    endTag("task");
  }
  
  private void printErr(String s) {
    this.err.print(s);
    if ((this.tagBits & 0x1) == 0 && this.log != null)
      this.log.print(s); 
  }
  
  private void printlnErr() {
    this.err.println();
    if ((this.tagBits & 0x1) == 0 && this.log != null)
      this.log.println(); 
  }
  
  private void printlnErr(String s) {
    this.err.println(s);
    if ((this.tagBits & 0x1) == 0 && this.log != null)
      this.log.println(s); 
  }
  
  private void printlnOut(String s) {
    this.out.println(s);
    if ((this.tagBits & 0x1) == 0 && this.log != null)
      this.log.println(s); 
  }
  
  public void printNewLine() {
    this.out.println();
  }
  
  private void printOut(char c) {
    this.out.print(c);
  }
  
  public void printStats() {
    boolean isTimed = ((this.main.timing & 0x1) != 0);
    if ((this.tagBits & 0x1) != 0)
      printTag("stats", null, true, false); 
    if (isTimed) {
      CompilerStats compilerStats = this.main.batchCompiler.stats;
      compilerStats.startTime = this.main.startTime;
      compilerStats.endTime = System.currentTimeMillis();
      logTiming(compilerStats);
    } 
    if (this.main.globalProblemsCount > 0)
      logProblemsSummary(this.main.globalProblemsCount, this.main.globalErrorsCount, this.main.globalWarningsCount, this.main.globalTasksCount); 
    if (this.main.exportedClassFilesCounter != 0 && (
      this.main.showProgress || isTimed || this.main.verbose))
      logNumberOfClassFilesGenerated(this.main.exportedClassFilesCounter); 
    if ((this.tagBits & 0x1) != 0)
      endTag("stats"); 
  }
  
  private void printTag(String name, HashMap params, boolean insertNewLine, boolean closeTag) {
    if (this.log != null)
      ((GenericXMLWriter)this.log).printTag(name, this.parameters, true, insertNewLine, closeTag); 
    this.parameters.clear();
  }
  
  public void setEmacs() {
    this.tagBits |= 0x2;
  }
  
  public void setLog(String logFileName) {
    Date date = new Date();
    DateFormat dateFormat = DateFormat.getDateTimeInstance(3, 1, Locale.getDefault());
    try {
      int index = logFileName.lastIndexOf('.');
      if (index != -1) {
        if (logFileName.substring(index).toLowerCase().equals(".xml")) {
          this.log = (PrintWriter)new GenericXMLWriter(new OutputStreamWriter(new FileOutputStream(logFileName, false), "UTF-8"), Util.LINE_SEPARATOR, true);
          this.tagBits |= 0x1;
          this.log.println("<!-- " + dateFormat.format(date) + " -->");
          this.log.println("<!DOCTYPE compiler PUBLIC \"-//Eclipse.org//DTD Eclipse JDT 3.2.004 Compiler//EN\" \"http://www.eclipse.org/jdt/core/compiler_32_004.dtd\">");
          this.parameters.put("name", this.main.bind("compiler.name"));
          this.parameters.put("version", this.main.bind("compiler.version"));
          this.parameters.put("copyright", this.main.bind("compiler.copyright"));
          printTag("compiler", this.parameters, true, false);
        } else {
          this.log = new PrintWriter(new FileOutputStream(logFileName, false));
          this.log.println("# " + dateFormat.format(date));
        } 
      } else {
        this.log = new PrintWriter(new FileOutputStream(logFileName, false));
        this.log.println("# " + dateFormat.format(date));
      } 
    } catch (FileNotFoundException fileNotFoundException) {
      throw new IllegalArgumentException(this.main.bind("configure.cannotOpenLog", logFileName));
    } catch (UnsupportedEncodingException unsupportedEncodingException) {
      throw new IllegalArgumentException(this.main.bind("configure.cannotOpenLogInvalidEncoding", logFileName));
    } 
  }
  
  private void startLoggingExtraProblems(int count) {
    this.parameters.put("problems", Integer.valueOf(count));
    printTag("extra_problems", this.parameters, true, false);
  }
  
  private void startLoggingProblems(int errors, int warnings) {
    this.parameters.put("problems", Integer.valueOf(errors + warnings));
    this.parameters.put("errors", Integer.valueOf(errors));
    this.parameters.put("warnings", Integer.valueOf(warnings));
    printTag("problems", this.parameters, true, false);
  }
  
  public void startLoggingSource(CompilationResult compilationResult) {
    if ((this.tagBits & 0x1) != 0) {
      ICompilationUnit compilationUnit = compilationResult.compilationUnit;
      if (compilationUnit != null) {
        char[] fileName = compilationUnit.getFileName();
        File f = new File(new String(fileName));
        if (fileName != null)
          this.parameters.put("path", f.getAbsolutePath()); 
        char[][] packageName = compilationResult.packageName;
        if (packageName != null)
          this.parameters.put(
              "package", 
              new String(CharOperation.concatWith(packageName, File.separatorChar))); 
        CompilationUnit unit = (CompilationUnit)compilationUnit;
        String destinationPath = unit.destinationPath;
        if (destinationPath == null)
          destinationPath = this.main.destinationPath; 
        if (destinationPath != null && destinationPath != "none")
          if (File.separatorChar == '/') {
            this.parameters.put("output", destinationPath);
          } else {
            this.parameters.put("output", destinationPath.replace('/', File.separatorChar));
          }  
      } 
      printTag("source", this.parameters, true, false);
    } 
  }
  
  public void startLoggingSources() {
    if ((this.tagBits & 0x1) != 0)
      printTag("sources", null, true, false); 
  }
  
  public void startLoggingTasks(int tasks) {
    if ((this.tagBits & 0x1) != 0) {
      this.parameters.put("tasks", Integer.valueOf(tasks));
      printTag("tasks", this.parameters, true, false);
    } 
  }
}
